"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var child_process_1 = require("child_process");
var chalk_1 = tslib_1.__importDefault(require("chalk"));
var cross_spawn_1 = tslib_1.__importDefault(require("cross-spawn"));
var open_1 = tslib_1.__importDefault(require("open"));
// https://github.com/sindresorhus/open#app
var OSX_CHROME = 'google chrome';
var Action;
(function (Action) {
    Action[Action["NONE"] = 0] = "NONE";
    Action[Action["BROWSER"] = 1] = "BROWSER";
    Action[Action["SCRIPT"] = 2] = "SCRIPT";
})(Action || (Action = {}));
function getBrowserEnv() {
    // Attempt to honor this environment variable.
    // It is specific to the operating system.
    // See https://github.com/sindresorhus/open#app for documentation.
    var value = process.env.BROWSER;
    var args = process.env.BROWSER_ARGS
        ? process.env.BROWSER_ARGS.split(' ')
        : [];
    var action;
    if (!value) {
        // Default.
        action = Action.BROWSER;
    }
    else if (value.toLowerCase().endsWith('.js')) {
        action = Action.SCRIPT;
    }
    else if (value.toLowerCase() === 'none') {
        action = Action.NONE;
    }
    else {
        action = Action.BROWSER;
    }
    return { action: action, value: value, args: args };
}
function executeNodeScript(scriptPath, url) {
    var extraArgs = process.argv.slice(2);
    var child = cross_spawn_1.default('node', [scriptPath].concat(extraArgs, [url]), {
        stdio: 'inherit',
    });
    child.on('close', function (code) {
        if (code !== 0) {
            console.log();
            console.log(chalk_1.default.red('The script specified as BROWSER environment variable failed.'));
            console.log(chalk_1.default.cyan(scriptPath) + " exited with code " + code + ".\n");
        }
    });
    return true;
}
function startBrowserProcess(browser, url, args) {
    // If we're on OS X, the user hasn't specifically
    // requested a different browser, we can try opening
    // Chrome with AppleScript. This lets us reuse an
    // existing tab when possible instead of creating a new one.
    var shouldTryOpenChromeWithAppleScript = process.platform === 'darwin' &&
        (typeof browser !== 'string' || browser === OSX_CHROME);
    if (shouldTryOpenChromeWithAppleScript) {
        try {
            // Try our best to reuse existing tab
            // on OS X Google Chrome with AppleScript
            child_process_1.execSync('ps cax | grep "Google Chrome"');
            child_process_1.execSync('osascript ../openChrome.applescript "' + encodeURI(url) + '"', {
                cwd: __dirname,
                stdio: 'ignore',
            });
            return true;
        }
        catch (err) {
            // Ignore errors.
        }
    }
    // Another special case: on OS X, check if BROWSER has been set to "open".
    // In this case, instead of passing `open` to `opn` (which won't work),
    // just ignore it (thus ensuring the intended behavior, i.e. opening the system browser):
    // https://github.com/facebook/create-react-app/pull/1690#issuecomment-283518768
    if (process.platform === 'darwin' && browser === 'open') {
        browser = undefined;
    }
    // If there are arguments, they must be passed as array with the browser
    if (typeof browser === 'string' && args.length > 0) {
        browser = [browser].concat(args);
    }
    // Fallback to open
    // (It will always open new tab)
    // tslint:disable-next-line no-try-promise
    try {
        var options = { app: browser, wait: false };
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        open_1.default(url, options).catch(function () { }); // Prevent `unhandledRejection` error.
        return true;
    }
    catch (err) {
        return false;
    }
}
/**
 * Reads the BROWSER environment variable and decides what to do with it. Returns
 * true if it opened a browser or ran a node.js script, otherwise false.
 */
function openBrowser(url) {
    var _a = getBrowserEnv(), action = _a.action, value = _a.value, args = _a.args;
    switch (action) {
        case Action.NONE:
            // Special case: BROWSER="none" will prevent opening completely.
            return false;
        case Action.SCRIPT:
            return executeNodeScript(value, url);
        case Action.BROWSER:
            return startBrowserProcess(value, url, args);
        default:
            throw new Error('Not implemented.');
    }
}
exports.openBrowser = openBrowser;
//# sourceMappingURL=browser.js.map